@@ -21,6 +21,9 @@ module Cachix.Client.OptionsParser
     -- * Pin options
     PinOptions (..),
 
+    -- * Doctor options
+    DoctorOptions (..),
+
     -- * Global options
     Flags (..),
 
@@ -79,6 +82,7 @@ data CachixCommand
   = AuthToken (Maybe Text)
   | Config Config.Command
   | Daemon DaemonCommand
+  | Doctor DoctorOptions
   | GenerateKeypair BinaryCacheName
   | Push PushArguments
   | Import PushOptions Text URI
@@ -168,6 +172,7 @@ data DaemonCommand
   | DaemonRun DaemonOptions PushOptions BinaryCacheName
   | DaemonStop DaemonOptions
   | DaemonWatchExec DaemonOptions PushOptions BinaryCacheName Text [Text]
+  | DaemonDoctor DaemonOptions
   deriving (Show)
 
 data DaemonOptions = DaemonOptions
@@ -182,6 +187,12 @@ data DaemonPushOptions = DaemonPushOptions
   }
   deriving (Show)
 
+data DoctorOptions = DoctorOptions
+  { doctorCacheName :: Maybe BinaryCacheName,
+    doctorStorePath :: Maybe Text
+  }
+  deriving (Show)
+
 -- | CLI parser entry point
 getOpts :: IO (Flags, CachixCommand)
 getOpts = do
@@ -206,6 +217,7 @@ commandParser =
     <|> storePathCommands
     <|> daemonCommands
     <|> deployCommands
+    <|> diagnosticCommands
   where
     configCommands =
       subparser $
@@ -260,6 +272,14 @@ commandParser =
             command "deploy" $ infoH deployCommand $ progDesc "Manage remote Nix-based systems with Cachix Deploy"
           ]
 
+    diagnosticCommands =
+      subparser $
+        fold
+          [ commandGroup "Diagnostic commands:",
+            hidden,
+            command "doctor" $ infoH doctorCommand $ progDesc "Check Cachix configuration and connectivity"
+          ]
+
 flagParser :: Config.ConfigPath -> Parser Flags
 flagParser defaultConfigPath =
   Flags <$> configPath <*> (host <|> hostname) <*> verbose
@@ -496,12 +516,16 @@ daemonSubCommand :: Parser DaemonCommand
 daemonSubCommand =
   subparser $
     fold
-      [ command "push" $ infoH daemonPush $ progDesc "Push store paths to the daemon",
+      [ command "doctor" $ infoH daemonDoctorParser $ progDesc "Check daemon health and connectivity",
+        command "push" $ infoH daemonPush $ progDesc "Push store paths to the daemon",
         command "run" $ infoH daemonRun $ progDesc "Launch the daemon",
         command "stop" $ infoH daemonStop $ progDesc "Stop the daemon and wait for any queued paths to be pushed",
         command "watch-exec" $ infoH daemonWatchExec $ progDesc "Run a command and upload any store paths built during its execution"
       ]
 
+daemonDoctorParser :: Parser DaemonCommand
+daemonDoctorParser = DaemonDoctor <$> daemonOptionsParser
+
 daemonPush :: Parser DaemonCommand
 daemonPush =
   DaemonPushPaths
@@ -595,6 +619,25 @@ daemonPushOptionsParser =
 deployCommand :: Parser CachixCommand
 deployCommand = DeployCommand <$> DeployOptions.parser
 
+doctorCommand :: Parser CachixCommand
+doctorCommand =
+  Doctor
+    <$> ( DoctorOptions
+            <$> cacheOption
+            <*> storePathArg
+        )
+  where
+    cacheOption =
+      optional . strOption $
+        long "cache"
+          <> metavar "CACHE-NAME"
+          <> help "Binary cache to check"
+
+    storePathArg =
+      optional . strArgument $
+        metavar "STORE-PATH"
+          <> help "Check if a specific store path or hash exists in the cache"
+
 watchExecCommand :: Parser CachixCommand
 watchExecCommand =
   WatchExec
