@@ -11,6 +11,7 @@ module Cachix.Daemon
   )
 where
 
+import Cachix.API qualified as API
 import Cachix.Client.Command.Push qualified as Command.Push
 import Cachix.Client.Config qualified as Config
 import Cachix.Client.Config.Orphans ()
@@ -18,6 +19,8 @@ import Cachix.Client.Env as Env
 import Cachix.Client.OptionsParser (DaemonOptions, PushOptions, daemonNarinfoQueryOptions)
 import Cachix.Client.OptionsParser qualified as Options
 import Cachix.Client.Push
+import Cachix.Client.Retry (retryHttp)
+import Cachix.Client.Servant (cachixClient)
 import Cachix.Daemon.EventLoop qualified as EventLoop
 import Cachix.Daemon.Listen as Listen
 import Cachix.Daemon.Log qualified as Log
@@ -42,6 +45,7 @@ import Katip qualified
 import Network.Socket qualified as Socket
 import Network.Socket.ByteString qualified as Socket.BS
 import Protolude hiding (bracket)
+import Servant.Client.Streaming (runClientM)
 import System.Environment (lookupEnv)
 import System.IO.Error (isResourceVanishedError)
 import System.Posix.Process (getProcessID)
@@ -89,7 +93,11 @@ new daemonEnv nixStore daemonOptions daemonLogHandle daemonPushOptions daemonCac
 
   daemonSubscriptionManagerThread <- newEmptyMVar
   daemonSubscriptionManager <- Subscription.newSubscriptionManager
-  let onPushEvent = Subscription.pushEvent daemonSubscriptionManager
+  let onPushEvent pushId event = do
+        Subscription.pushEvent daemonSubscriptionManager pushId event
+        case Types.eventMessage event of
+          Types.PushFinished -> Subscription.queueUnsubscribe daemonSubscriptionManager pushId
+          _ -> return ()
 
   let pushParams = Push.newPushParams nixStore (clientenv daemonEnv) daemonBinaryCache daemonPushSecret daemonPushOptions
   daemonPushManager <- PushManager.newPushManagerEnv daemonPushOptions (daemonNarinfoQueryOptions daemonOptions) pushParams onPushEvent daemonLogger
@@ -144,12 +152,20 @@ run daemon = fmap join <$> runDaemon daemon $ do
     ReceivedMessage socketId clientMsg ->
       case clientMsg of
         ClientPushRequest pushRequest -> do
-          maybePushId <- queueJob pushRequest
-          case maybePushId of
-            Just pushId -> when (Protocol.subscribeToUpdates pushRequest) $ do
+          -- Create push job upfront so we can subscribe before queuing
+          pushJob <- liftIO $ PushManager.newPushJob pushRequest
+          let pushId = PushManager.pushId pushJob
+
+          Katip.katipAddContext (Katip.sl "push_id" pushId) $ do
+            when (Protocol.subscribeToUpdates pushRequest) $ do
               chan <- liftIO $ subscribe daemon (Just pushId)
-              void $ liftIO $ Async.async $ publishToClient socketId chan daemon
-            Nothing -> Katip.logFM Katip.ErrorS "Failed to queue push request"
+              publisherThread <- liftIO $ Async.async $ publishToClient socketId chan daemon
+              SocketStore.addPublisherThread socketId pushId publisherThread daemonClients
+
+            -- Now add the job
+            success <- queueJob pushJob
+            unless success $
+              Katip.logFM Katip.ErrorS "Failed to queue push request"
         ClientStop -> do
           DaemonEnv {daemonOptions = opts, daemonClients = clients} <- ask
           if Options.daemonAllowRemoteStop opts
@@ -157,6 +173,32 @@ run daemon = fmap join <$> runDaemon daemon $ do
             else do
               let errorMsg = "Remote stop is disabled on this daemon"
               SocketStore.sendAll socketId (Protocol.newMessage (Protocol.DaemonError (Protocol.UnsupportedCommand errorMsg))) clients
+        ClientDiagnosticsRequest -> do
+          -- Get push secret to check for signing key and auth token
+          let pushParams = PushManager.pmPushParams daemonPushManager
+              pushSecret = pushParamsSecret pushParams
+              hasSigningKey = case pushSecret of
+                PushSigningKey {} -> True
+                PushToken {} -> False
+              maybeToken = getAuthTokenFromPushSecret pushSecret
+          -- Verify auth by making an API call
+          authResult <- case maybeToken of
+            Nothing -> pure $ Left "No auth token configured"
+            Just token -> do
+              res <- liftIO $ retryHttp $ (`runClientM` clientenv daemonEnv) $ API.getCache cachixClient token daemonCacheName
+              pure $ case res of
+                Right _ -> Right ()
+                Left err -> Left (show err)
+          let diagnostics =
+                Protocol.DaemonDiagnostics
+                  { Protocol.diagCacheName = daemonCacheName,
+                    Protocol.diagCacheUri = BinaryCache.uri daemonBinaryCache,
+                    Protocol.diagCachePublic = BinaryCache.isPublic daemonBinaryCache,
+                    Protocol.diagHasSigningKey = hasSigningKey,
+                    Protocol.diagAuthOk = isRight authResult,
+                    Protocol.diagError = either Just (const Nothing) authResult
+                  }
+          SocketStore.sendAll socketId (Protocol.newMessage (Protocol.DaemonDiagnosticsResult diagnostics)) daemonClients
         _ -> return ()
     ShutdownGracefully -> do
       Katip.logFM Katip.InfoS "Shutting down daemon..."
@@ -225,10 +267,10 @@ installSignalHandlers = do
         threadDelay (15 * 1000 * 1000) -- 15 seconds
         throwTo mainThreadId ExitSuccess
 
-queueJob :: Protocol.PushRequest -> Daemon (Maybe PushRequestId)
-queueJob pushRequest = do
+queueJob :: PushManager.PushJob -> Daemon Bool
+queueJob pushJob = do
   DaemonEnv {daemonPushManager} <- ask
-  liftIO $ PushManager.runPushManager daemonPushManager (PushManager.addPushJob pushRequest)
+  liftIO $ PushManager.runPushManager daemonPushManager (PushManager.addPushJob pushJob)
 
 subscribe :: DaemonEnv -> Maybe PushRequestId -> IO (TMChan PushEvent)
 subscribe daemonEnv maybePushId = do
@@ -243,14 +285,16 @@ subscribeAll daemonEnv = subscribe daemonEnv Nothing
 
 -- Publish messages from the channel to the client over the socket
 publishToClient :: SocketId -> TMChan PushEvent -> DaemonEnv -> IO ()
-publishToClient socketId chan daemonEnv = do
-  forever $ do
-    msg <- atomically $ readTMChan chan
-    case msg of
-      Nothing -> return ()
-      Just evt -> do
-        let daemonMsg = DaemonPushEvent evt
-        SocketStore.sendAll socketId (Protocol.newMessage daemonMsg) (daemonClients daemonEnv)
+publishToClient socketId chan daemonEnv = go
+  where
+    go = do
+      msg <- atomically $ readTMChan chan
+      case msg of
+        Nothing -> return ()
+        Just evt -> do
+          let daemonMsg = DaemonPushEvent evt
+          SocketStore.sendAll socketId (Protocol.newMessage daemonMsg) (daemonClients daemonEnv)
+          go
 
 -- | Print the daemon configuration to the log.
 printConfiguration :: Daemon ()
