@@ -1,5 +1,6 @@
 module Cachix.Daemon.Types.SocketStore (Socket (..), SocketId, SocketStore (..)) where
 
+import Cachix.Daemon.Types.PushEvent (PushRequestId)
 import Control.Concurrent.STM.TVar (TVar)
 import Data.HashMap.Strict (HashMap)
 import Data.UUID (UUID)
@@ -9,7 +10,8 @@ import Protolude
 data Socket = Socket
   { socketId :: SocketId,
     socket :: Network.Socket,
-    handlerThread :: Async ()
+    handlerThread :: Async (),
+    publisherThreads :: TVar (HashMap PushRequestId (Async ()))
   }
 
 instance Eq Socket where
